#!/usr/bin/env bash

set -o errexit

# #############################################################################
# Option management
# #############################################################################

TEMP=`getopt -o d:t: -- "$@"`

if [ $? != 0 ] ; then echo "Terminating..." >&2 ; exit 1 ; fi

usage () {
    echo 'Set suitable UNIX permissions to binary files imported into Comptoir'
    echo 'NOTE: this must be run as a user with *sudo* privileges'
    echo ''
    echo 'usage: $0 [OPTIONS]...'
    echo ''
    echo '  -d <comptoir-srv-dir>   (MANDATORY) Absolute directory to Comptoir-srv, *without* trailing slash, eg "/home/comptoir/comptoir-srv"'
    echo ''
    exit 2
}

# Note the double quotes around $TEMP: they are essential!
eval set -- "${TEMP}"

declare COMPTOIR_SRV_DIR=

while true; do
    case "$1" in
        -d ) COMPTOIR_SRV_DIR="$2"; shift 2 ;;
        -- ) shift; break ;;
        * ) break ;;
    esac
done

# check mandatory options
if [ "${COMPTOIR_SRV_DIR}" = "" ]
then
    echo ''
    echo 'Mandatory option is missing'
    echo ''
    usage
fi

# #############################################################################
# Variables
# #############################################################################

DIR_WEBROOT_FILES="${COMPTOIR_SRV_DIR}/webroot/img/files"

# #############################################################################
# Actual action
# #############################################################################

sudo chown -R www-data.comptoir "${DIR_WEBROOT_FILES}/"
find "${DIR_WEBROOT_FILES}" -type f -exec sudo chmod 664 {} \;
find "${DIR_WEBROOT_FILES}" -type d -exec sudo chmod 775 {} \;
