<?php
namespace App\Test\TestCase\Controller;


use Cake\I18n\Time;
use Cake\TestSuite\IntegrationTestCase;
use Cake\ORM\TableRegistry;

/**
 * App\Controller\LicensesController Test Case
 */
class PagesControllerTest extends IntegrationTestCase
{

    /**
     * Fixtures
     *
     * @var array
     */
    public $fixtures = [
        'app.users',
        'app.user_types',
        'app.licenses',
        'app.licence_types',
        'app.softwares',
        'app.relationships',
        'app.relationships_softwares_users',
        'app.relationship_types',
        'app.softwares_statistics',
        'app.reviews'
    ];

    public function setUp()
    {
        parent::setUp(); // TODO: Change the autogenerated stub
        //Needs : Softwares and Users to run a research
        $this->Users = TableRegistry::get('Users');
        $this->Softwares = TableRegistry::get('Softwares');

    }

    /**
     *
     */
    public function testSearchOnUser()
    {

        $this->get("api/v1/pages/search.json?search=company");
        // Vérifie que le code de réponse est 200
        $this->assertResponseOk();
        $expected = [
            "softwares" => [],
            "users" => [
                [
                    "id" => 2,
                    "username" => "Avencall Company",
                    "created" => "2017-02-14T15:02:50+00:00",
                    "logo_directory" => TESTS . "TestFiles" . DS . "Users/photo/id2/avatar",
                    "url" => "http://www.avencall.com/",
                    "user_type_id" => 2,
                    "description" => "Avencall transforme l’industrie des télécommunications d’entreprise et son modèle économique en donnant un accès universel à des solutions de téléphonie qui assurent à ses clients liberté, évolutivité et créativité au service de leur métier. Une réussite rendue possible par la collaboration et le savoir-faire de ses équipes.",
                    "modified" => "2017-02-14T15:02:50+00:00",
                    "role" => "user",
                    "photo" => "correctUserAvatar.jpg",
                    "user_type" => [
                        "id" => 2,
                        "name" => "Company",
                        "created" => "2017-02-14T15:02:50+00:00",
                        "modified" => "2017-02-14T15:02:50+00:00",
                        "cd" => "Company"
                    ]
                ]
            ]
        ];
        $expected = json_encode($expected, JSON_PRETTY_PRINT);
        $this->assertEquals($expected, $this->_response->body());

    }

    /**
     * Test Find by type when type is specified
     */
    public function testFindByTypeNotSpecified()
    {
        $this->get("api/v1/pages/search.json?search=");
        // Vérifie que le code de réponse est 200
        $this->assertResponseOk();

        $expected = [
            "softwares" => [
                [
                    "average_review" => 3,
                    "id" => 1,
                    "softwarename" => "Asalae",
                    "url_repository" => "http://www.fake-repo-asalae.git",
                    "description" => "Asalae description",
                    "licence_id" => 1,
                    "created" => "2017-02-14T15:02:48+00:00",
                    "modified" => "2017-02-14T15:02:48+00:00",
                    "logo_directory" => TESTS . "TestFiles" . DS . "Softwares/photo/id1/avatar",
                    "photo" => "correctSoftwareLogo.jpg",
                    "url_website" => "http://www.fake-asalae.com",
                    "web_site_url" => null,
                    "reviews" => [
                        [
                            "id" => 1,
                            "comment" => "Commentaire sur le soft id = 1 par utilisateur id = 1",
                            "title" => "Titre de commentaire",
                            "created" => "2017-02-14T15:02:47+00:00",
                            "user_id" => 1,
                            "software_id" => 1,
                            "evaluation" => 3,
                            "modified" => "2017-02-14T15:02:47+00:00"
                        ],
                        [
                            "id" => 2,
                            "comment" => "Commentaire sur le soft id = 1 par utilisateur id = 2",
                            "title" => "Titre de commentaire 2 ",
                            "created" => "2017-02-14T15:02:47+00:00",
                            "user_id" => 2,
                            "software_id" => 1,
                            "evaluation" => 5,
                            "modified" => "2017-02-14T15:02:47+00:00"
                        ],
                        [
                            "id" => 3,
                            "comment" => "Commentaire sur le soft id = 1 par utilisateur id = 3",
                            "title" => "Titre de commentaire 3",
                            "created" => "2017-02-14T15:02:47+00:00",
                            "user_id" => 3,
                            "software_id" => 1,
                            "evaluation" => 2,
                            "modified" => "2017-02-14T15:02:47+00:00"
                        ]
                    ]
                ],
                [
                    "average_review" => 3,
                    "id" => 2,
                    "softwarename" => "Lutèce",
                    "url_repository" => "http://www.fake-repo-lutece.git",
                    "description" => "Lutèce description",
                    "licence_id" => 1,
                    "created" => "2017-02-14T15:02:48+00:00",
                    "modified" => "2017-02-14T15:02:48+00:00",
                    "logo_directory" => TESTS . "TestFiles" . DS . "Softwares/photo/id2/avatar",
                    "photo" => "correctSoftwareLogo.jpg",
                    "url_website" => "http://www.fake-lutece.com",
                    "web_site_url" => null,
                    "reviews" => [
                        [
                            "id" => 4,
                            "comment" => "Commentaire sur le soft id = 2 par utilisateur id = 3",
                            "title" => "Titre de commentaire 4",
                            "created" => "2017-02-14T15:02:47+00:00",
                            "user_id" => 3,
                            "software_id" => 2,
                            "evaluation" => 3,
                            "modified" => "2017-02-14T15:02:47+00:00"
                        ]
                    ]
                ],
                [
                    "average_review" => 0,
                    "id" => 3,
                    "softwarename" => "Soft 3",
                    "url_repository" => "http://www.fake-repo-soft3.git",
                    "description" => "Lutèce description",
                    "licence_id" => 1,
                    "created" => "2017-02-14T15:02:48+00:00",
                    "modified" => "2017-02-14T15:02:48+00:00",
                    "logo_directory" => TESTS . "TestFiles" . DS . "Softwares/photo/id3/avatar",
                    "photo" => "correctSoftwareLogo.jpg",
                    "url_website" => "http://www.fake-soft3.com",
                    "web_site_url" => null,
                    "reviews" => []
                ]
            ],
            "users" => [
                [
                    "id" => 3,
                    "username" => "Administration",
                    "created" => Time::createFromTimestamp(1487084570),
                    "logo_directory" => TESTS . "TestFiles" . DS . "Users/photo/id3/avatar",
                    "url" => "http://www.avencall.com/",
                    "user_type_id" => 3,
                    "description" => "description",
                    "modified" => Time::createFromTimestamp(1487084570),
                    "role" => "user",
                    "photo" => "correctUserAvatar.jpg",
                    "user_type" => [
                        'id' => 3,
                        'name' => 'Administration',
                        'created' => Time::createFromTimestamp(1487084570),
                        'modified' => Time::createFromTimestamp(1487084570),
                        'cd' => 'Administration'
                    ],
                ],
                [
                    "id" => 1,
                    "username" => "Admin User",
                    "created" => Time::createFromTimestamp(1487084570),
                    "logo_directory" => TESTS . "TestFiles" . DS . "Users/photo/id1/avatar",
                    "url" => "http://www.adullact.org/",
                    "user_type_id" => 1,
                    "description" => "ADULLACT a pour objectifs de soutenir et coordonner l action des Administrations et Collectivités territoriales dans le but de promouvoir, développer et maintenir un patrimoine de logiciels libres utiles aux missions de service public.",
                    "modified" => Time::createFromTimestamp(1487084570),
                    "role" => "admin",
                    "photo" => "correctUserAvatar.jpg",
                    "user_type" => [
                        'id' => 1,
                        'name' => 'Association',
                        'created' => Time::createFromTimestamp(1487084570),
                        'modified' => Time::createFromTimestamp(1487084570),
                        'cd' => 'Asso'
                    ],
                ],
                [
                    "id" => 2,
                    "username" => "Avencall Company",
                    "created" => Time::createFromTimestamp(1487084570),
                    "logo_directory" => TESTS . "TestFiles" . DS . "Users/photo/id2/avatar",
                    "url" => "http://www.avencall.com/",
                    "user_type_id" => 2,
                    "description" => "Avencall transforme l’industrie des télécommunications d’entreprise et son modèle économique en donnant un accès universel à des solutions de téléphonie qui assurent à ses clients liberté, évolutivité et créativité au service de leur métier. Une réussite rendue possible par la collaboration et le savoir-faire de ses équipes.",
                    "modified" => Time::createFromTimestamp(1487084570),
                    "role" => "user",
                    "photo" => "correctUserAvatar.jpg",
                    "user_type" => [
                        'id' => 2,
                        'name' => 'Company',
                        'created' => Time::createFromTimestamp(1487084570),
                        'modified' => Time::createFromTimestamp(1487084570),
                        'cd' => 'Company'
                    ],
                ],

                [
                    "id" => 4,
                    "username" => "Person",
                    "created" => Time::createFromTimestamp(1487084570),
                    "logo_directory" => TESTS . "TestFiles" . DS . "Users/photo/id4/avatar",
                    "url" => "http://www.avencall.com/",
                    "user_type_id" => 4,
                    "description" => "",
                    "modified" => Time::createFromTimestamp(1487084570),
                    "role" => "user",
                    "photo" => "correctUserAvatar.jpg",
                    "user_type" => [
                        'id' => 4,
                        'name' => 'Person',
                        'created' => Time::createFromTimestamp(1487084570),
                        'modified' => Time::createFromTimestamp(1487084570),
                        'cd' => 'Person'
                    ],
                ]
            ]
        ];
        $expected = json_encode($expected, JSON_PRETTY_PRINT);
        $this->assertEquals($expected, $this->_response->body());
    }


    public function testSearchUserHadReview (){
        $this->get("api/v1/pages/search.json?search=&hadReview=true");
        // Vérifie que le code de réponse est 200
        $this->assertResponseOk();


        $expected = [
            "softwares" => [
                [
                    "average_review" => 3,
                    "id" => 1,
                    "softwarename" => "Asalae",
                    "url_repository" => "http://www.fake-repo-asalae.git",
                    "description" => "Asalae description",
                    "licence_id" => 1,
                    "created" => "2017-02-14T15:02:48+00:00",
                    "modified" => "2017-02-14T15:02:48+00:00",
                    "logo_directory" => TESTS . "TestFiles" . DS . "Softwares/photo/id1/avatar",
                    "photo" => "correctSoftwareLogo.jpg",
                    "url_website" => "http://www.fake-asalae.com",
                    "web_site_url" => null,
                    "reviews" => [
                        [
                            "id" => 1,
                            "comment" => "Commentaire sur le soft id = 1 par utilisateur id = 1",
                            "title" => "Titre de commentaire",
                            "created" => "2017-02-14T15:02:47+00:00",
                            "user_id" => 1,
                            "software_id" => 1,
                            "evaluation" => 3,
                            "modified" => "2017-02-14T15:02:47+00:00"
                        ],
                        [
                            "id" => 2,
                            "comment" => "Commentaire sur le soft id = 1 par utilisateur id = 2",
                            "title" => "Titre de commentaire 2 ",
                            "created" => "2017-02-14T15:02:47+00:00",
                            "user_id" => 2,
                            "software_id" => 1,
                            "evaluation" => 5,
                            "modified" => "2017-02-14T15:02:47+00:00"
                        ],
                        [
                            "id" => 3,
                            "comment" => "Commentaire sur le soft id = 1 par utilisateur id = 3",
                            "title" => "Titre de commentaire 3",
                            "created" => "2017-02-14T15:02:47+00:00",
                            "user_id" => 3,
                            "software_id" => 1,
                            "evaluation" => 2,
                            "modified" => "2017-02-14T15:02:47+00:00"
                        ]
                    ]
                ],
                [
                    "average_review" => 3,
                    "id" => 2,
                    "softwarename" => "Lutèce",
                    "url_repository" => "http://www.fake-repo-lutece.git",
                    "description" => "Lutèce description",
                    "licence_id" => 1,
                    "created" => "2017-02-14T15:02:48+00:00",
                    "modified" => "2017-02-14T15:02:48+00:00",
                    "logo_directory" => TESTS . "TestFiles" . DS . "Softwares/photo/id2/avatar",
                    "photo" => "correctSoftwareLogo.jpg",
                    "url_website" => "http://www.fake-lutece.com",
                    "web_site_url" => null,
                    "reviews" => [
                        [
                            "id" => 4,
                            "comment" => "Commentaire sur le soft id = 2 par utilisateur id = 3",
                            "title" => "Titre de commentaire 4",
                            "created" => "2017-02-14T15:02:47+00:00",
                            "user_id" => 3,
                            "software_id" => 2,
                            "evaluation" => 3,
                            "modified" => "2017-02-14T15:02:47+00:00"
                        ]
                    ]
                ],
                [
                    "average_review" => 0,
                    "id" => 3,
                    "softwarename" => "Soft 3",
                    "url_repository" => "http://www.fake-repo-soft3.git",
                    "description" => "Lutèce description",
                    "licence_id" => 1,
                    "created" => "2017-02-14T15:02:48+00:00",
                    "modified" => "2017-02-14T15:02:48+00:00",
                    "logo_directory" => TESTS . "TestFiles" . DS . "Softwares/photo/id3/avatar",
                    "photo" => "correctSoftwareLogo.jpg",
                    "url_website" => "http://www.fake-soft3.com",
                    "web_site_url" => null,
                    "reviews" => []
                ]
            ],
            "users" => [
                [
                    "id" => 3,
                    "username" => "Administration",
                    "created" => Time::createFromTimestamp(1487084570),
                    "logo_directory" => TESTS . "TestFiles" . DS . "Users/photo/id3/avatar",
                    "url" => "http://www.avencall.com/",
                    "user_type_id" => 3,
                    "description" => "description",
                    "modified" => Time::createFromTimestamp(1487084570),
                    "role" => "user",
                    "photo" => "correctUserAvatar.jpg",
                    "user_type" => [
                        'id' => 3,
                        'name' => 'Administration',
                        'created' => Time::createFromTimestamp(1487084570),
                        'modified' => Time::createFromTimestamp(1487084570),
                        'cd' => 'Administration'
                    ],
                ],
                [
                    "id" => 1,
                    "username" => "Admin User",
                    "created" => Time::createFromTimestamp(1487084570),
                    "logo_directory" => TESTS . "TestFiles" . DS . "Users/photo/id1/avatar",
                    "url" => "http://www.adullact.org/",
                    "user_type_id" => 1,
                    "description" => "ADULLACT a pour objectifs de soutenir et coordonner l action des Administrations et Collectivités territoriales dans le but de promouvoir, développer et maintenir un patrimoine de logiciels libres utiles aux missions de service public.",
                    "modified" => Time::createFromTimestamp(1487084570),
                    "role" => "admin",
                    "photo" => "correctUserAvatar.jpg",
                    "user_type" => [
                        'id' => 1,
                        'name' => 'Association',
                        'created' => Time::createFromTimestamp(1487084570),
                        'modified' => Time::createFromTimestamp(1487084570),
                        'cd' => 'Asso'
                    ],
                ],
                [
                    "id" => 2,
                    "username" => "Avencall Company",
                    "created" => Time::createFromTimestamp(1487084570),
                    "logo_directory" => TESTS . "TestFiles" . DS . "Users/photo/id2/avatar",
                    "url" => "http://www.avencall.com/",
                    "user_type_id" => 2,
                    "description" => "Avencall transforme l’industrie des télécommunications d’entreprise et son modèle économique en donnant un accès universel à des solutions de téléphonie qui assurent à ses clients liberté, évolutivité et créativité au service de leur métier. Une réussite rendue possible par la collaboration et le savoir-faire de ses équipes.",
                    "modified" => Time::createFromTimestamp(1487084570),
                    "role" => "user",
                    "photo" => "correctUserAvatar.jpg",
                    "user_type" => [
                        'id' => 2,
                        'name' => 'Company',
                        'created' => Time::createFromTimestamp(1487084570),
                        'modified' => Time::createFromTimestamp(1487084570),
                        'cd' => 'Company'
                    ],
                ],
            ]
        ];
        $expected = json_encode($expected, JSON_PRETTY_PRINT);
        $this->assertEquals($expected, $this->_response->body());
    }


    public function testSearchUserisUserOf (){
        $this->get("api/v1/pages/search.json?search=&isUserOf=true");
        // Vérifie que le code de réponse est 200
        $this->assertResponseOk();


        $expected = [
            "softwares" => [
                [
                    "average_review" => 3,
                    "id" => 1,
                    "softwarename" => "Asalae",
                    "url_repository" => "http://www.fake-repo-asalae.git",
                    "description" => "Asalae description",
                    "licence_id" => 1,
                    "created" => "2017-02-14T15:02:48+00:00",
                    "modified" => "2017-02-14T15:02:48+00:00",
                    "logo_directory" => TESTS . "TestFiles" . DS . "Softwares/photo/id1/avatar",
                    "photo" => "correctSoftwareLogo.jpg",
                    "url_website" => "http://www.fake-asalae.com",
                    "web_site_url" => null,
                    "reviews" => [
                        [
                            "id" => 1,
                            "comment" => "Commentaire sur le soft id = 1 par utilisateur id = 1",
                            "title" => "Titre de commentaire",
                            "created" => "2017-02-14T15:02:47+00:00",
                            "user_id" => 1,
                            "software_id" => 1,
                            "evaluation" => 3,
                            "modified" => "2017-02-14T15:02:47+00:00"
                        ],
                        [
                            "id" => 2,
                            "comment" => "Commentaire sur le soft id = 1 par utilisateur id = 2",
                            "title" => "Titre de commentaire 2 ",
                            "created" => "2017-02-14T15:02:47+00:00",
                            "user_id" => 2,
                            "software_id" => 1,
                            "evaluation" => 5,
                            "modified" => "2017-02-14T15:02:47+00:00"
                        ],
                        [
                            "id" => 3,
                            "comment" => "Commentaire sur le soft id = 1 par utilisateur id = 3",
                            "title" => "Titre de commentaire 3",
                            "created" => "2017-02-14T15:02:47+00:00",
                            "user_id" => 3,
                            "software_id" => 1,
                            "evaluation" => 2,
                            "modified" => "2017-02-14T15:02:47+00:00"
                        ]
                    ]
                ],
                [
                    "average_review" => 3,
                    "id" => 2,
                    "softwarename" => "Lutèce",
                    "url_repository" => "http://www.fake-repo-lutece.git",
                    "description" => "Lutèce description",
                    "licence_id" => 1,
                    "created" => "2017-02-14T15:02:48+00:00",
                    "modified" => "2017-02-14T15:02:48+00:00",
                    "logo_directory" => TESTS . "TestFiles" . DS . "Softwares/photo/id2/avatar",
                    "photo" => "correctSoftwareLogo.jpg",
                    "url_website" => "http://www.fake-lutece.com",
                    "web_site_url" => null,
                    "reviews" => [
                        [
                            "id" => 4,
                            "comment" => "Commentaire sur le soft id = 2 par utilisateur id = 3",
                            "title" => "Titre de commentaire 4",
                            "created" => "2017-02-14T15:02:47+00:00",
                            "user_id" => 3,
                            "software_id" => 2,
                            "evaluation" => 3,
                            "modified" => "2017-02-14T15:02:47+00:00"
                        ]
                    ]
                ],
                [
                    "average_review" => 0,
                    "id" => 3,
                    "softwarename" => "Soft 3",
                    "url_repository" => "http://www.fake-repo-soft3.git",
                    "description" => "Lutèce description",
                    "licence_id" => 1,
                    "created" => "2017-02-14T15:02:48+00:00",
                    "modified" => "2017-02-14T15:02:48+00:00",
                    "logo_directory" => TESTS . "TestFiles" . DS . "Softwares/photo/id3/avatar",
                    "photo" => "correctSoftwareLogo.jpg",
                    "url_website" => "http://www.fake-soft3.com",
                    "web_site_url" => null,
                    "reviews" => []
                ]
            ],
            "users" => [
                [
                    "id" => 3,
                    "username" => "Administration",
                    "created" => Time::createFromTimestamp(1487084570),
                    "logo_directory" => TESTS . "TestFiles" . DS . "Users/photo/id3/avatar",
                    "url" => "http://www.avencall.com/",
                    "user_type_id" => 3,
                    "description" => "description",
                    "modified" => Time::createFromTimestamp(1487084570),
                    "role" => "user",
                    "photo" => "correctUserAvatar.jpg",
                    "user_type" => [
                        'id' => 3,
                        'name' => 'Administration',
                        'created' => Time::createFromTimestamp(1487084570),
                        'modified' => Time::createFromTimestamp(1487084570),
                        'cd' => 'Administration'
                    ],
                ]
            ]
        ];
        $expected = json_encode($expected, JSON_PRETTY_PRINT);
        $this->assertEquals($expected, $this->_response->body());
    }

    /**
     *
     */
    public function testSearchOnSotfware()
    {

        $this->get("api/v1/pages/search.json?search=Soft");
        // Vérifie que le code de réponse est 200
        $this->assertResponseOk();


        $expected = [
            "softwares" => [
                [
                    "average_review" => 0,
                    "id" => 3,
                    "softwarename" => "Soft 3",
                    "url_repository" => "http://www.fake-repo-soft3.git",
                    "description" => "Lutèce description",
                    "licence_id" => 1,
                    "created" => "2017-02-14T15:02:48+00:00",
                    "modified" => "2017-02-14T15:02:48+00:00",
                    "logo_directory" => TESTS . "TestFiles" . DS . "Softwares/photo/id3/avatar",
                    "photo" => "correctSoftwareLogo.jpg",
                    "url_website" => "http://www.fake-soft3.com",
                    "web_site_url" => null,
                    "reviews" => [],
                ]
            ],
            "users" => []
        ];
        $expected = json_encode($expected, JSON_PRETTY_PRINT);
        $this->assertEquals($expected, $this->_response->body());

    }


    /**
     *The research => 'description' should return a software and an user
     */
    public function testSearchOnSotfwareAndUser()
    {


        $this->post("api/v1/pages/search.json", ["search" => "description"]);
        // Vérifie que le code de réponse est 200
        $this->assertResponseOk();

        $body = json_decode($this->_response->body());

        $this->assertNotEmpty($body->softwares);
        $this->assertNotEmpty($body->users);

    }


    /**
     * Test the search function
     * The research with "DesCRIpTIoN" OR "description" shoul have the same result
     */
    public function testSearchCaseInsensitive()
    {


        $this->post("api/v1/pages/search.json", ["search" => "DesCRIpTIoN"]);
        $this->assertResponseOk();
        $bodyCaseInsensitive = json_decode($this->_response->body());

        $this->post("api/v1/pages/search.json", ["search" => "description"]);
        $this->assertResponseOk();
        $bodyCaseSensitive = json_decode($this->_response->body());

        $this->assertEquals($bodyCaseInsensitive, $bodyCaseSensitive);
    }

    /**
     *Test the search function
     * The research with accent characters or non-accent characters should return the same result.
     */
    public function testSearchAccentInsensitive()
    {

        $this->get("api/v1/pages/search.json?description");
        $this->assertResponseOk();
        $bodyCaseInsensitive = json_decode($this->_response->body());

        $this->get("api/v1/pages/search.json?déscription");
        $this->assertResponseOk();
        $bodyAccentSensitive = json_decode($this->_response->body());

        $this->assertEquals($bodyCaseInsensitive, $bodyAccentSensitive);
    }
}
