<?php

namespace App\Test\TestCase\Controller\Api\V1;


use Cake\ORM\TableRegistry;
use Cake\Event\EventList;

/**
 * App\Controller\UsersController Test Case
 * @property \App\Model\Table\UsersTable $Users
 */
class UsersControllerTest extends ApiIntegrationTestCase
{

    /**
     * Fixtures
     *
     * @var array
     */
    public $fixtures = [
        'app.users',
        'app.user_types',
        'app.licenses',
        'app.licence_types',
        'app.reviews',
        'app.softwares',
        'app.screenshots',
        'app.relationships',
        'app.relationship_types',
        'app.softwares_statistics',
        'app.relationships_softwares_users'
    ];

    /**
     * @var array for all rules determined by the validationDefault method.
     */
    public $rules;

    public $correctFile;
    public $wrongFile;
    public $exceedFile;

    public $Users;
    public $userTypesArray;

    public function setUp()
    {
        parent::setUp(); // TODO: Change the autogenerated stub

        $this->Users = TableRegistry::get('Users');
        $this->Users->eventManager()->setEventList(new EventList());

        $config = TableRegistry::exists('UserTypes') ? [] : ['className' => 'App\Model\Table\UserTypesTable'];
        $this->userTypes = TableRegistry::get('usersType', $config);
        $this->userTypesArray = $this->userTypes->find("list")->toArray();


        $this->correctFile = [
            'name' => 'correctAvatarLogo.jpg',
            'tmp_name' => TESTS . "TestFiles" . DS . "Common" . DS . "correctAvatarLogo.jpg",
            'type' => 'image/jpeg',
            'size' => 8407,
            'error' => 0
        ];
        $this->exceedFile = [
            'name' => 'exceedPhpSizeLimitUserAvatar.jpg',
            'tmp_name' => TESTS . "TestFiles" . DS . "Common" . DS . 'exceedPhpSizeLimitUserAvatar.jpg',
            'type' => 'image/jpeg',
            'size' => 0,
            'error' => 1
        ];
        $this->wrongFile = [
            'name' => 'wrongAvatarLogo.jpg',
            'tmp_name' => TESTS . "TestFiles" . DS . "Common" . DS . 'wrongAvatarLogo.jpg',
            'type' => 'image/jpeg',
            'size' => 1132550,
            'error' => 0
        ];


        $this->rules = [
            "id" => [
                "presenceRequired" => false,
                "allowEmpty" => "create"
            ],
            "username" => [
                "presenceRequired" => "create",
                "allowEmpty" => false
            ],
            "logo_directory" => [
                "presenceRequired" => false,
                "allowEmpty" => true
            ],
            "url" => [
                "presenceRequired" => false,
                "allowEmpty" => true
            ],
            "description" => [
                "presenceRequired" => false,
                "allowEmpty" => true
            ],
            "role" => [
                "presenceRequired" => "create",
                "allowEmpty" => false,
                "notEquals" => [
                    "rule" => "comparison",
                    "message" => null,
                    "limit" => "!="
                ],
            ],
            "password" => [
                "presenceRequired" => "create",
                "allowEmpty" => false,
                "compare" => [
                    "rule" => "compareWith",
                    "message" => "Password field and confirm-password field should be equals.",
                    "limit" => "confirm_password"
                ]
            ],
            "email" => [
                "presenceRequired" => "create",
                "allowEmpty" => false,
                "email" => [
                    "rule" => "email",
                    "message" => "The email is invalid",
                    "limit" => false
                ]
            ],
            "photo" => [
                "presenceRequired" => false,
                "allowEmpty" => true,
                "UnderPhpSizeLimit" => [
                    "rule" => "isUnderPhpSizeLimit",
                    "message" => "The file can not be uploaded",
                    "limit" => null
                ],
                "fileFileUpload" => [
                    "rule" => "isFileUpload",
                    "message" => null,
                    "limit" => null
                ],
                "fileCompletedUpload" => [
                    "rule" => "isCompletedUpload",
                    "message" => null,
                    "limit" => null
                ],
                "file" => [
                    "rule" => "mimeType",
                    "message" => "This format is not allowed please use one in this list: JPEG, PNG, GIF, SVG.",
                    "limit" => [
                        "image/jpeg",
                        "image/png",
                        "image/gif",
                        "image/svg+xml"
                    ]
                ],
                "fileBelowMaxSize" => [
                    "rule" => "isBelowMaxSize",
                    "message" => "The maximum weight allowed is 1MB, please use a file less than 1MB.",
                    "limit" => 1048576
                ],
                "fileBelowMaxHeight" => [
                    "rule" => "isBelowMaxHeight",
                    "message" => "The size of your image is too big, please use an image fitting a 350x350px size.",
                    "limit" => 350
                ],
                "fileBelowMaxWidth" => [
                    "rule" => "isBelowMaxWidth",
                    "message" => "The size of your image is too big, please use an image fitting a 350x350px size.",
                    "limit" => 350
                ],

            ]
        ];
    }


    /**
     * Test add method
     * @return void
     */
    public function testAdd()
    {

        $user = [
            'username' => 'Toto',
            'logo_directory' => '',
            'url' => 'url',
            'user_type_id' => 1,
            'description' => 'A description',
            'role' => 'admin',
            'password' => 'passwd',
            'confirm_password' => 'passwd',
            'email' => 'toto@adullact.org',
            'photo' => $this->correctFile,
        ];

        $this->post('api/v1/users.json', $user);
        // Vérifie que le code de réponse est 200
        $this->assertResponseOk();
        $this->assertContains("Success", $this->_response->body());


        $this->assertEventFired('Model.User.created', $this->Users->eventManager());

        //Check in files directory
        $this->assertFileExists(WWW_ROOT . 'img' . DS . "files/Users/photo/5/avatar/" . $this->correctFile["name"]);

        //Delete the file after testing
        unlink(WWW_ROOT . 'img' . DS . "files/Users/photo/5/avatar/" . $this->correctFile["name"]);
        $this->assertFileNotExists(WWW_ROOT . 'img' . DS . "files/Users/photo/5/avatar/" . $this->correctFile["name"]);

    }

    /**
     * Test add method using a wrong image
     * Should return an array of errors containing all errors for the photo's field
     * @return void
     */
    public function testAddWrongImageSize()
    {

        $user = [
            'username' => 'Toto',
            'logo_directory' => '',
            'url' => 'url',
            'user_type_id' => 1,
            'description' => 'A description',
            'role' => 'User',
            'password' => 'passwd',
            'confirm_password' => 'passwd',
            'email' => 'toto@adullact.org',
            'photo' => $this->wrongFile,
            'modified' => "2016-09-23 09:01:19+0909",
            'created' => "2016-09-23 09:01:19+0909",
        ];

        $this->post('api/v1/users.json', $user);


        // Vérifie que le code de réponse est 200
        $this->assertResponseOk();

        unset($user["password"]);
        unset($user["email"]);
        unset($user["photo"]); // into errors[];

        $expected = [
            "user" => $user,
            "userTypes" => $this->userTypesArray,
            "rules" => $this->rules,
            'message' => "Error",
            "errors" => [
                "photo" => [
                    "fileBelowMaxSize" => "The maximum weight allowed is 1MB, please use a file less than 1MB.",
                    "fileBelowMaxHeight" => "The size of your image is too big, please use an image fitting a 350x350px size.",
                ]
            ],
        ];

        $expected = json_encode($expected, JSON_PRETTY_PRINT);
        $this->assertEquals($expected, $this->_response->body());
    }


    /**
     * Test add method
     * @return void
     */
    public function testAddWrongExceedPhpLimitSize()
    {

        $user = [
            'username' => 'Toto',
            'logo_directory' => '',
            'url' => 'url',
            'user_type_id' => 1,
            'description' => 'A description',
            'role' => 'admin',
            'password' => 'passwd',
            'confirm_password' => 'passwd',
            'email' => 'toto@adullact.org',
            'photo' => $this->exceedFile,
            'modified' => "2016-09-23 09:01:19+0909",
            'created' => "201ll
            6-09-23 09:01:19+0909",
        ];

        $this->post('api/v1/users/add.json', $user);
        // Vérifie que le code de réponse est 200
        $this->assertResponseOk();

        unset($user["password"]);
        unset($user["email"]);
        unset($user["photo"]); // into errors[];

        $expected = [
            "user" => $user,
            "userTypes" => $this->userTypesArray,
            "rules" => $this->rules,
            'message' => "Error",
            "errors" => [
                "photo" => [
                    "UnderPhpSizeLimit" => "The file can not be uploaded",
                    "fileBelowMaxHeight" => "The size of your image is too big, please use an image fitting a 350x350px size.",

                ]
            ],
        ];

        $expected = json_encode($expected, JSON_PRETTY_PRINT);
        $this->assertEquals($expected, $this->_response->body());
    }

    /**
     * Test add method
     * @return void
     */
    public function testAddAnExistingUserName()
    {

        $user = [
            'username' => 'Administration',
            'logo_directory' => '',
            'url' => 'url',
            'user_type_id' => 1,
            'description' => 'A description',
            'role' => 'admin',
            'password' => 'passwd',
            'confirm_password' => 'passwd',
            'email' => 'toto@adullact.org',
            'photo' => null,
            'modified' => "2016-09-23 09:01:19+0909",
            'created' => "2016-09-23 09:01:19+0909",
        ];

        $this->post('api/v1/users/add.json', $user);

        $this->assertResponseOk();

        unset($user["password"]);
        unset($user["email"]);

        $expected = [
            "user" => $user,
            "userTypes" => $this->userTypesArray,
            "rules" => $this->rules,
            'message' => "Error",
            "errors" => [
                "username" => [
                    "_isUnique" => "A user with this name already exists.",
                ]
            ],
        ];

        $expected = json_encode($expected, JSON_PRETTY_PRINT);
        $this->assertEquals($expected, $this->_response->body());

    }


    /**
     * Test add method
     * @return void
     */
    public function testAddAnExistingEmail()
    {
        $user = [
            'username' => 'Toto',
            'logo_directory' => '',
            'url' => 'url',
            'user_type_id' => 1,
            'description' => 'A description',
            'role' => 'admin',
            'password' => 'passwd',
            'confirm_password' => 'passwd',
            'email' => 'fake-email-administration@comptoir-du-libre.org',
            'photo' => null,
            'modified' => "2016-09-23 09:01:19+0909",
            'created' => "2016-09-23 09:01:19+0909",
        ];

        $this->post('api/v1/users/add.json', $user);

        $this->assertResponseOk();

        unset($user["password"]);
        unset($user["email"]);

        $expected = [
            "user" => $user,
            "userTypes" => $this->userTypesArray,
            "rules" => $this->rules,
            'message' => "Error",
            "errors" => [
                "email" => [
                    "_isUnique" => "A user with this email already exists.",
                ]
            ],
        ];

        $expected = json_encode($expected, JSON_PRETTY_PRINT);
        $this->assertEquals($expected, $this->_response->body());

    }


    /**
     */
    public function testAddWhenPasswordIsNotEqualsConfirmPassword()
    {
        $user = [
            'username' => 'Toto',
            'logo_directory' => '',
            'url' => 'url',
            'user_type_id' => 1,
            'description' => 'A description',
            'role' => 'user',
            'password' => 'passwd',
            'confirm_password' => 'NotEqualsToPassword',
            'email' => 'toto@adullact.org',
            'photo' => null,
            'modified' => "2016-09-23 09:01:19+0909",
            'created' => "2016-09-23 09:01:19+0909",
        ];

        $this->post('api/v1/users/add.json', $user);

        // Vérifie que le code de réponse est 200
        $this->assertResponseOk();

        unset($user["password"]);
        unset($user["email"]);

        $expected = [
            "user" => $user,
            "userTypes" => $this->userTypesArray,
            "rules" => $this->rules,
            'message' => "Error",
            "errors" => [
                "password" => [
                    "compare" => "Password field and confirm-password field should be equals."
                ]
            ],
        ];

        $expected = json_encode($expected, JSON_PRETTY_PRINT);
        $this->assertEquals($expected, $this->_response->body());
    }


    /**
     * Test edit method
     *
     * @return void
     */
    public function testEditAnotherAcount()
    {

        $this->session([
            'Auth' => [
                'User' => [
                    'id' => 2,
                    "username" => "Avencall Company",
                    "user_type_id" => 2,
                    "role" => "User",
                ]
            ]
        ]);

        $data = [
            'username' => 'Toto',
            'modified' => "2016-09-27 15:20:32+0303",
        ];

        $this->put('api/v1/users/1.json', $data);

        $this->assertEquals($this->_exception->getCode(), 403);

    }


    /**
     * Test edit method
     *
     * @return void
     */
    public function testEditUserName()
    {

        $this->session([
            'Auth' => [
                'User' => [
                    'id' => 3,
                    "user_type_id" => 3,
                ]
            ]
        ]);


        $user = [
            'username' => 'Toto',
            'modified' => time(),
        ];

        $this->put('api/v1/users/3.json', $user);

        $this->assertResponseOk();

        $this->assertContains("Success", $this->_response->body());
        $this->assertContains($user["username"], $this->_response->body());
    }

    /**
     * Test edit method
     *
     * @return void
     */
    public function testEditEmail()
    {

        $this->session([
            'Auth' => [
                'User' => [
                    'id' => 3,
                    "user_type_id" => 3,
                ]
            ]
        ]);


        $user = [
            'email' => 'fake-email-administration-edit@comptoir-du-libre.org',
            'modified' => time(),
        ];

        $this->put('api/v1/users/3.json', $user);

        $this->assertResponseOk();

        $this->assertContains("Success", $this->_response->body());

        $query = $this->Users->find("all")->where(["email"=>'fake-email-administration-edit@comptoir-du-libre.org']);
        $this->assertEquals(1, $query->count());

    }

    /**
     * Test edit method
     *
     * @return void
     */
    public function testEditNotConnected()
    {
        $data = [
            'username' => 'Toto',
            'modified' => "2016-09-27 15:20:32+0303",
        ];

        $this->put('api/v1/users/edit/2.json', $data);
        $this->assertResponseCode(302);
    }

    /**
     * Test view method
     *
     * @return void
     */
    public function testView()
    {

        $this->get('api/v1/users/2.json');
        $this->assertResponseOk();

        $user = [
            "user" => [
                "id" => 2,
                "username" => "Avencall Company",
                "created" => "2017-02-14T15:02:50+00:00",
                "logo_directory" => TESTS . "TestFiles" . DS . "Users/photo/id2/avatar",
                "url" => "http://www.avencall.com/",
                "user_type_id" => 2,
                "description" => "Avencall transforme l’industrie des télécommunications d’entreprise et son modèle économique en donnant un accès universel à des solutions de téléphonie qui assurent à ses clients liberté, évolutivité et créativité au service de leur métier. Une réussite rendue possible par la collaboration et le savoir-faire de ses équipes.",
                "modified" => "2017-02-14T15:02:50+00:00",
                "role" => "user",
                "photo" => "correctUserAvatar.jpg",
                "providerforsoftwares" => [],
                "usedsoftwares" => [],
                "reviews" => [
                    [
                        "id" => 2,
                        "comment" => "Commentaire sur le soft id = 1 par utilisateur id = 2",
                        "title" => "Titre de commentaire 2 ",
                        "created" => "2017-02-14T15:02:47+00:00",
                        "user_id" => 2,
                        "software_id" => 1,
                        "evaluation" => 5,
                        "modified" => "2017-02-14T15:02:47+00:00",
                        "software"=> [
                            "id"=> 1,
                            "softwarename"=> "Asalae",
                            "logo_directory"=> TESTS . "TestFiles" . DS . "Softwares/photo/id1/avatar",
                            "photo"=> "correctSoftwareLogo.jpg",
                            "average_review"=> 0,
                            "tag_string"=> "",
                        ],
                    ],
                ],
                "user_type" => [
                    "id" => 2,
                    "name" => "Company",
                    "created" => "2017-02-14T15:02:50+00:00",
                    "modified" => "2017-02-14T15:02:50+00:00",
                    "cd" => "Company"
                ],
            ],

        ];

        $this->assertEquals(json_encode($user, JSON_PRETTY_PRINT), $this->_response->body());
    }

    /**
     * Test view method
     *
     * @return void
     */
    public function testViewNotFound()
    {
        $this->get('api/v1/users/999.json');
        $this->assertResponseCode('404');
    }


    /**
     * Test register method
     */
    public function testRegisterFailOnWrongPassword(){
        $this->login("fake-email-company@comptoir-du-libre.org", "good-");


        $this->assertFalse($this->_controller->Auth->identify());
    }

    /**
     * Test register method
     */
    public function testRegisterFailOnWrongUserName(){
        $this->login("fake-email-fake@comptoir-du-libre.org", "good-password");

        $this->assertFalse($this->_controller->Auth->identify());
    }

    public function testContact (){

        $this->login("fake-email-administration@comptoir-du-libre.org","good-password");


        $this->get('api/v1/users/contact/1.json');
        $this->assertResponseOk();


        // var use in the title of the page and the fieldset
        $this->assertContains("\"username\": \"Admin User\"",$this->_response->body());

        $email = [
          "subject" => "Test title",
          "content" => "test content",
          "email" => "fake-email-administration@comptoir-du-libre.org"
        ];

        $this->post('/users/contact/1',$email);

        $this->assertResponseOk();
    }
}
